// --------------------------------------------------------------------
// Copyright (c) 2005 by Terasic Technologies Inc. 
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// --------------------------------------------------------------------
//           
//                     Terasic Technologies Inc
//                     356 Fu-Shin E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------
//
// Major Functions:	DE2 LTM module Timing control and color pattern 
//					generator
//
// --------------------------------------------------------------------
//
// Revision History :
// --------------------------------------------------------------------
//   Ver  :| Author            		:| Mod. Date :| Changes Made:
//   V1.0 :| Johnny Fan				:| 07/06/30  :| Initial Revision
// --------------------------------------------------------------------
module lcd_timing_controller		(
						clk, 				// LCD display clock
						reset_n, 			// systen reset
						read,
						write,
						chipselect,
						address,
						readdata,
						writedata,
						//LCD SIDE
						oHD,				// LCD Horizontal sync 
						oVD,				// LCD Vertical sync 	
						oDEN,				// LCD Data Enable
						oLCD_R,				// LCD Red color data 
						oLCD_G,             // LCD Green color data  
						oLCD_B,             // LCD Blue color data  
						iDISPLAY_MODE
						);
//============================================================================
// PARAMETER declarations
//============================================================================
parameter H_LINE = 1056;
parameter V_LINE = 525;
parameter Hsync_Blank = 216;
parameter Hsync_Front_Porch = 40;
parameter Vertical_Back_Porch = 35;
parameter Vertical_Front_Porch = 10;
//===========================================================================
// PORT declarations
//===========================================================================
input	wire		clk;   
input	wire		reset_n;
input	wire		read;
input	wire		write;
input	wire		chipselect;
input	wire [4:0]address;
output wire [15:0]readdata;
input	wire [15:0]writedata;
output	[7:0]	oLCD_R;		
output  [7:0]	oLCD_G;
output  [7:0]	oLCD_B;
output			oHD;
output			oVD;
output			oDEN;
input	[1:0]	iDISPLAY_MODE;	

//=============================================================================
// REG/WIRE declarations
//=============================================================================
reg		[10:0]  x_cnt;  
reg		[9:0]	y_cnt; 
wire	[7:0]	mred;
wire	[7:0]	mgreen;
wire	[7:0]	mblue; 
wire			display_area;
reg				mhd;
reg				mvd;
reg				mden;
reg				oHD;
reg				oVD;
reg				oDEN;
reg		[7:0]	oLCD_R;
reg		[7:0]	oLCD_G;	
reg		[7:0]	oLCD_B;	
wire	[1:0]	msel;	
reg		[7:0]	red_1;
reg 	[7:0]	green_1;
reg 	[7:0]	blue_1;
reg 	[7:0]	graycnt;
reg		[7:0]	pattern_data;
reg [31:0] div;
wire clk_ltm;
assign clk_ltm = div[0]; // 25 Mhz
wire [3:0] area;
reg [23:0]oRGB_DATA; //information containing mole state
reg [2:0]mole0;
reg [2:0]mole1;
reg [2:0]mole2;
reg [2:0]mole3;
reg [2:0]mole4;
reg [2:0]mole5;
reg [2:0]mole6;
reg [2:0]mole7;
reg [2:0]mole8;
reg [2:0]mole9;

//=============================================================================
// Structural coding
//=============================================================================

//clock Generate 25MHz
always @(posedge clk)
	begin
		div <= div+1;
	end
			
//Detect which area you are in
assign area = ((x_cnt>(Hsync_Blank-1+200)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-400))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+160))
						))  ? 4'b0 : ; //mole0
assign area = ((x_cnt>(Hsync_Blank-1+400)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-200))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+160))
						))  ? 4'b1 :;	//mole1
assign area = ((x_cnt>(Hsync_Blank-1+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+160))
						))  ? 4'b2 : ;//mole2
assign area = ((x_cnt>(Hsync_Blank-1+200)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-400))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+160))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+320))
						))  ? 4'b3 : ;	//mole3
assign area = ((x_cnt>(Hsync_Blank-1+400)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-200))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+160))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+320))
						))  ? 4'b4 : ; //mole4
assign area = ((x_cnt>(Hsync_Blank-1+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+160))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+320))
						))  ? 4'b5 : ;//mole5
assign area = ((x_cnt>(Hsync_Blank-1+200)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-400))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+320))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+480))
						))  ? 4'b6 : ;	//mole6
assign area = ((x_cnt>(Hsync_Blank-1+400)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-200))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+320))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+480))
						))  ? 4'b7 : ; //mole7
assign area = ((x_cnt>(Hsync_Blank-1+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+320))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+480))
						))  ? 4'b8 : ;//mole8
assign area = ((x_cnt>(Hsync_Blank-1)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch+200))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+80))
						))  ? 4'd9 : ;//area 9 Time
assign area = ((x_cnt>(Hsync_Blank-1)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch+200))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+80))
						))  ? 4'd10 : ;//area 10 life
assign area = ((x_cnt>(Hsync_Blank-1)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch+200))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+80))
						))  ? 4'd11 : ;//area 11 score
assign area = ((x_cnt>(Hsync_Blank-1)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch+200))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+80))
						))  ? 4'd12 : ;//area 12 start/pause
assign area = ((x_cnt>(Hsync_Blank-1)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch+200))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+80))
						))  ? 4'd13 : ;//area 13 exit
assign area = ((x_cnt>(Hsync_Blank-1)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch+200))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch+80))
						))  ? 4'd14 : ;//area 14 label
//area 15 is the logo of the team						

// This signal indicate the lcd display area .
assign	display_area = ((x_cnt>(Hsync_Blank-1)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch))
						))  ? 1'b1 : 1'b0;


///////////////////////// x  y counter  and lcd hd generator //////////////////
always@(posedge clk_ltm or negedge reset_n)
	begin
		if (!reset_n)
		begin
			x_cnt <= 11'd0;	
			mhd  <= 1'd0;  	
		end	
		else if (x_cnt == (H_LINE-1))
		begin
			x_cnt <= 11'd0;
			mhd  <= 1'd0;
		end	   
		else
		begin
			x_cnt <= x_cnt + 11'd1;
			mhd  <= 1'd1;
		end	
	end

always@(posedge clk_ltm or negedge reset_n)
	begin
		if (!reset_n)
			y_cnt <= 10'd0;
		else if (x_cnt == (H_LINE-1))
		begin
			if (y_cnt == (V_LINE-1))
				y_cnt <= 10'd0;
			else
				y_cnt <= y_cnt + 10'd1;	
		end
	end
////////////////////////////// touch panel timing //////////////////

always@(posedge clk_ltm  or negedge reset_n)
	begin
		if (!reset_n)
			mvd  <= 1'b1;
		else if (y_cnt == 10'd0)
			mvd  <= 1'b0;
		else
			mvd  <= 1'b1;
	end			

always@(posedge clk_ltm  or negedge reset_n)
	begin
		if (!reset_n)
			mden  <= 1'b0;
		else if (display_area)
			mden  <= 1'b1;
		else
			mden  <= 1'b0;
	end			

// Test Generator
//always@(posedge clk_ltm or negedge reset_n)
//	begin
//		if(!reset_n)
//			begin
//			red_1 <= 8'd0;
//			green_1 <= 8'd0;
//			blue_1 <= 8'd0;
//			end
//		else if(iDISPLAY_MODE == 0)
//			begin
//			red_1 <= 8'd255;
//			green_1 <= 8'd0;
//			blue_1 <= 8'd0;
//			end
//		else if(iDISPLAY_MODE == 1)
//			begin
//			red_1 <= 8'd0;
//			green_1 <= 8'd255;
//			blue_1 <= 8'd0;
//			end
//		else
//			begin
//			red_1 <= 8'd0;
//			green_1 <= 8'd0;
//			blue_1 <= 8'd255;
//			end
//	end

//Test
always@(posedge clk_ltm or negedge reset_n)
	begin
		case(area)
			4'd0: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd1: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd2: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd3: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd4: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd5: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd6: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd7: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd8: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd9: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd10: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd11: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd12: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd13: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd14: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			4'd15: red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd0;
			
			
	end
assign mred = red_1;
assign mgreen = green_1;
assign mblue  = blue_1;

//////////////RGB color patten generator ///////
//
//
//assign	msel		=	(y_cnt<200)					?	2'b01	:
//						(y_cnt>=200	&& y_cnt<360)	?	2'b10	:
//													    2'b00	;
//always@(posedge clk_ltm  or negedge reset_n)
//	begin
//		if (!reset_n)
//			pattern_data	<=	8'h00;
//		else if((x_cnt>(Hsync_Blank-1))&&(x_cnt<(H_LINE-Hsync_Front_Porch)))
//			begin
//				if(msel==1)
//					begin
//						pattern_data <= pattern_data + 1;		
//						red_1 <= pattern_data;
//						green_1 <= 0;
//						blue_1 <= 0;
//					end	
//				else if (msel==2)
//					begin
//						pattern_data <= pattern_data + 1;
//						red_1 <= 0;
//						green_1 <= pattern_data;
//						blue_1 <= 0;
//					end	
//				else if (msel==0)
//					begin
//						pattern_data <= pattern_data + 1;
//						red_1 <= 0;
//						green_1 <= 0;
//						blue_1 <= pattern_data;
//					end	
//			end	
//				else
//					pattern_data<=	8'h00;	
//	end	
//
////////////////gray level  patten generator ///////
//
//always@(posedge clk_ltm  or negedge reset_n)
//	begin
//		if (!reset_n)
//			graycnt <= 0;
//		else if((x_cnt>(Hsync_Blank-1))&&(x_cnt<(H_LINE-Hsync_Front_Porch))) 
//			graycnt <= graycnt + 1;	
//		else
//			graycnt <= 0;		
//end	

////////////// displayed color pattern selection //////////////
//
//assign mred    = (iDISPLAY_MODE == 2'b11)?  8'hff:  
//				 (iDISPLAY_MODE == 2'b10)?  8'h7f: 
//			     (iDISPLAY_MODE == 2'b01)?  red_1:
//										    graycnt; 
//
//assign mgreen  = (iDISPLAY_MODE == 2'b11)?  8'hff:
//			     (iDISPLAY_MODE == 2'b10)?  8'h7f:
//				 (iDISPLAY_MODE == 2'b01)?  green_1:
//				  				            graycnt;
//				 
//assign mblue   = (iDISPLAY_MODE == 2'b11)?  8'hff:
//			     (iDISPLAY_MODE == 2'b10)?  8'h7f:
//				 (iDISPLAY_MODE == 2'b01)?  blue_1:
//				  				            graycnt;


always@(posedge clk_ltm or negedge reset_n)
	begin
		if (!reset_n)
			begin
				oHD	<= 1'd0;
				oVD	<= 1'd0;
				oDEN <= 1'd0;
				oLCD_R <= 8'd0;
				oLCD_G <= 8'd0;
				oLCD_B <= 8'd0;
			end
		else
			begin
				oHD	<= mhd;
				oVD	<= mvd;
				oDEN <= display_area;
				oLCD_R <= mred;
				oLCD_G <= mgreen;
				oLCD_B <= mblue;
			end		
	end
						
endmodule











