module panel_ctrl(
					clk_ltm,
					reset_n, //top level connection
					x_cnt, // input from lcd controlelr
					y_cnt, // input from lcd controller
					iPANELCTRL, // command from nios containing 32 bits
					oOVERRIDE,//override output to the lcd_controller
					oADDR, //the output address to the lcd controller
					//ostate
					);
//===============
//PARAMETER
//==================
parameter BASE_ADDR = 89024;
parameter H_LINE = 1056;
parameter V_LINE = 525;
parameter Hsync_Blank = 216;
parameter Hsync_Front_Porch = 40;
parameter Vertical_Back_Porch = 35;
parameter Vertical_Front_Porch = 10; 
//=================================
//IO PORT
//=================================
input clk_ltm;
input reset_n;
input [10:0] x_cnt;
input [9:0] y_cnt;
input [31:0] iPANELCTRL;
output oOVERRIDE;
output [17:0] oADDR;
//output [3:0] ostate;
//===================================
//REG and WIRE
//====================================
wire oOVERRIDE;
wire [17:0] oADDR; // output address to lcd controller
wire [15:0] area; // 15 areas in total
wire [17:0] iADDR [0:5]; // address from the 6 peripheral areas
wire [5:0] override; //6 bit override signal from 6 area
wire [3:0] life;
wire [3:0] score0;
wire [3:0] score1;
wire [3:0] score2;
wire [3:0] state; // init->start-> pause

assign		state = iPANELCTRL[3:0];
assign		life =  iPANELCTRL[7:4];
assign		score0 = iPANELCTRL[11:8];
assign		score1 = iPANELCTRL[15:12];
assign		score2 = iPANELCTRL[19:16];
assign 		ostate = state;

Timmer Timmer1(
			.clk_ltm(clk_ltm),
			.reset_n(reset_n),
			.x_cnt(x_cnt),
			.y_cnt(y_cnt),
			.state(state),
			.oOVERRIDE(override[0]),
			.oADDR(iADDR[0])
			);
			
Life Life1(
			.clk_ltm(clk_ltm),
			.reset_n(reset_n),
			.x_cnt(x_cnt),
			.y_cnt(y_cnt),
			.life(life),
			.oOVERRIDE(override[1]),
			.oADDR(iADDR[1])
			);
			
Score Score1(
			.clk_ltm(clk_ltm),
			.reset_n(reset_n),
			.x_cnt(x_cnt),
			.y_cnt(y_cnt),
			.iScore0(score0),
			.iScore1(score1),
			.iScore2(score2),
			.oOVERRIDE(override[2]),
			.oADDR(iADDR[2])
			);

quit quit1 (
			.iclk(clk_ltm),
			.ireset_n(reset_n),
			.ix_cnt(x_cnt),
			.iy_cnt(y_cnt),
			.oOVERRIDE(override[5]),
			.oADDR(iADDR[5]));
			
start start1 (
			.iclk(clk_ltm),
			.ireset_n(reset_n),
			.ix_cnt(x_cnt),
			.iy_cnt(y_cnt),
			.oOVERRIDE(override[3]),
			.oADDR(iADDR[3])
			);

pause pause1 (
			.iclk(clk_ltm),
			.ireset_n(reset_n),
			.ix_cnt(x_cnt),
			.iy_cnt(y_cnt),
			.oOVERRIDE(override[4]),
			.oADDR(iADDR[4])
			);

		

//Detect which area you are in// 0~14 different areas//offset corrected
assign area[0] = ((x_cnt>(Hsync_Blank-2+400)&& //>214
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-200))&& //< 1015
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-320))
						))  ? 1'b1 : 1'b0 ; //mole0
assign area[1] = ((x_cnt>(Hsync_Blank-2+200)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-400))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-320))
						))  ? 1'b1 : 1'b0;//mole1
assign area[2] = ((x_cnt>(Hsync_Blank-2)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-600))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-320))
						))  ? 1'b1 : 1'b0 ;//mole2
assign area[3] = ((x_cnt>(Hsync_Blank-2+400)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-200))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+160))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-160))
						))  ? 1'b1 : 1'b0 ;	//mole3
assign area[4] = ((x_cnt>(Hsync_Blank-2+200)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-400))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+160))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-160))
						))  ? 1'b1 : 1'b0 ; //mole4
assign area[5] = ((x_cnt>(Hsync_Blank-2)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-600))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+160))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-160))
						))  ? 1'b1 : 1'b0 ;//mole5
assign area[6] = ((x_cnt>(Hsync_Blank-2+400)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-200))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+320))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch))
						))  ? 1'b1 : 1'b0 ;	//mole6
assign area[7] = ((x_cnt>(Hsync_Blank-2+200)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-400))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+320))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch))
						))  ? 1'b1 : 1'b0; //mole7
assign area[8] = ((x_cnt>(Hsync_Blank-2)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-600))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+320))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch))
						))  ? 1'b1 : 1'b0 ;//mole8
assign area[9] = ((x_cnt>(Hsync_Blank-2+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-400))
						))  ? 1'b1 : 1'b0 ;//area 9 Time
assign area[10] = ((x_cnt>(Hsync_Blank-2+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+80))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-320))
						))  ? 1'b1 : 1'b0 ;//area 10 life
assign area[11] = ((x_cnt>(Hsync_Blank-2+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+160))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-240))
						))  ? 1'b1 : 1'b0 ;//area 11 score
assign area[12] = ((x_cnt>(Hsync_Blank-2+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+240))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-160))
						))  ? 1'b1 : 1'b0 ;//area 12 start/pause
assign area[13] = ((x_cnt>(Hsync_Blank-2+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+320))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-80))
						))  ? 1'b1 : 1'b0 ;//area 13 exit
assign area[14] = ((x_cnt>(Hsync_Blank-2+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+400))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch))
						))  ? 1'b1 : 1'b0 ;//area 14 label


assign oOVERRIDE = override[0] || override[1] || override[2] || override[3] || override[4] || override[5];
assign oADDR = (area[9] && override[0]) ? iADDR [0] :
					(area[10] && override[1]) ? iADDR [1] :
					(area[11] && override[2]) ? iADDR [2] :
					(area[12] && override[3]) ? iADDR [3] :
					(area[13] && override[4]) ? iADDR [4] :
					(area[14] && override[5]) ? iADDR [5] : BASE_ADDR;
endmodule

